//***************************************************************************//
//*                                                                         *//
//* Unit:    xml_mainUnit                                                   *//
//* Author:  Dushkin R.V.                                                   *//
//* Date:    24.04.01                                                       *//
//*                                                                         *//
//*  This is the main unit of "XMLParser" project. Contains main class des- *//
//* cription and all necessary methods for XML Interface.                   *//
//*                                                                         *//
//***************************************************************************//
unit xml_mainUnit;

//-[ INTERFACE ]---------------------------------------------------------------
interface

uses
  Sysutils,
  xml_node,
  xml_attribute,
  xml_parameters,
  xml_functions,
  xml_types;

type TXMLParser = class (TObject)
  private
    fFileName      : String;         // File Name of parsed XML-document.
    fXMLErrorCode  : TXMLErrorCode;  // Returned ErrorCode after parsing process.
    fXMLRoot       : TXMLNode;       // Pointer to XML-Root.
    fXMLParameters : TXMLParameters; // Pointer to XML Parameters.

    function ParseFile : TXMLErrorCode;
    function Validate  : TXMLErrorCode;

  public
    constructor Create;
    destructor Destroy; override;

    procedure setFileName (const iFileName : String);
    procedure SaveFile (const iFileName : String);

    function FindTagByName (tagName : String) : TXMLNode;

  published
    property FileName      : String read fFileName write setFileName;
    property XMLErrorCode  : TXMLErrorCode read fXMLErrorCode;
    property XMLRoot       : TXMLNode read fXMLRoot;
    property XMLParameters : TXMLParameters read fXMLParameters;

end;

//-[ IMPLEMENTATION ]----------------------------------------------------------
implementation

constructor TXMLParser.Create;
begin
  inherited Create;
  fFileName := '';
  fXMLErrorCode := ecOk;
  fXMLRoot := TXMLNode.Create;
  fXMLParameters := TXMLParameters.Create;
end;

destructor TXMLParser.Destroy;
begin
  fXMLRoot.Free;
  fXMLParameters.Free;
  inherited Destroy;
end;

{$IOCHECKS OFF}
function TXMLParser.ParseFile : TXMLErrorCode;
var
  commentStart : Integer;
  tempStr      : String;
  trimmedStr   : String;
  xmlText      : String;
  xmlFile      : TextFile;
begin
  xmlText := '';
  AssignFile (xmlFile, fFileName);
  Reset (xmlFile);
  if (IOResult = 0) then
  begin
    while (not Eof (xmlFile)) do
    begin
      repeat
        ReadLn (xmlFile, tempStr);
      until (tempStr <> '');
      trimmedStr := TrimSpaces (tempStr);
      if (trimmedStr[1] = '<') then
      begin
        xmlText := xmlText + trimmedStr;
      end
      else
      begin
        xmlText := xmlText + ' ' + trimmedStr;
      end;
    end;
    CloseFile (xmlFile);

    { DONE 5 -oDushkin R.V. -cFunction : Create the cycle to delete comments from input string. }
    commentStart := Pos ('<!--', xmlText);
    while (commentStart > 0) do
    begin
      Delete (xmlText, commentStart, Pos ('-->', xmlText) - commentStart + 3);
      commentStart := Pos ('<!--', xmlText);
    end;

    { DONE 5 -oDushkin R.V. -cFunction : Create parsing cycle. }
    //-------------------------------------------------------------------------
    // *** This is XML Parsing Cycle ***
    //-------------------------------------------------------------------------
    // At first XML String is parsed for XML parameters (version, encoding,
    // etc.). Second, the rest of XML String is parsed for XML Tree itself. If
    // after this process XML String will be empty, all is Ok, and the XML Tree
    // will be validated (using DTD if any).
    //-------------------------------------------------------------------------
    Result := fXMLParameters.ParseString (xmlText);
    if (Result = ecOk) then
    begin
      Result := fXMLRoot.ParseString (0, xmlText);
    end;
    //-------------------------------------------------------------------------

  end
  else
  begin
    Result := ecNoFile;
    CloseFile (xmlFile);
  end;
  if (Result = ecOk) then
  begin
    if (xmlText = '') then
    begin
      Result := Validate;
    end
    else
    begin
      Result := ecNotValid;
    end;
  end;
end;
{$IOCHECKS ON}

function TXMLParser.Validate : TXMLErrorCode;
begin
  Result := ecOk;
end;

procedure TXMLParser.setFileName (const iFileName : String);
begin
  if (iFileName <> '') then
  begin
    fFileName := iFileName;
    fXMLErrorCode := ParseFile;
  end
  else
  begin
    fXMLErrorCode := ecInvalidFileName;
  end;
end;

function TXMLParser.FindTagByName (tagName : String) : TXMLNode;
begin
  if (fXMLRoot.TagName = tagName) then
  begin
    Result := fXMLRoot;
  end
  else
  begin
    Result := fXMLRoot.FindTagByName (tagName);
  end;
end;

procedure TXMLParser.SaveFile (const iFileName : String);
var
  xmlFile : TextFile;
begin
  fFileName := iFileName;
  Assign (xmlFile, fFileName);
  Rewrite (xmlFile);                   
  fXMLParameters.SaveToFile (xmlFile);
  fXMLRoot.SaveToFile (xmlFile);
  CloseFile (xmlFile);
end;

//-[ END ]---------------------------------------------------------------------
end.
