//***************************************************************************//
//*                                                                         *//
//* Unit:    xml_parameters                                                 *//
//* Author:  Dushkin R.V.                                                   *//
//* Date:    27.04.01                                                       *//
//*                                                                         *//
//*  This module contains the class, which describes "Node" object of XML   *//
//* document.                                                               *//
//*                                                                         *//
//***************************************************************************//
unit xml_parameters;

//-[ INTERFACE ]---------------------------------------------------------------
interface

uses
  classes,
  xml_attribute,
  xml_docType,
  xml_functions,
  xml_types;

type TXMLParameters = class (TObject)
  private
    fXMLVersion       : String;      // Version of XML Document.
    fXMLEncoding      : String;      // Encoding of XML Document.
    fXMLLanguage      : String;      // Language of XML Document.
    fXMLStandalone    : String;      // Standalone of XML Document.
    fXMLOtherAttrs    : TList;       // Other attributes of XML Document.
    fXMLStyleSheet    : String;      // Link to StyleSheet file of XML Document.
    fXMLDocType       : TXMLDocType; // DocType (DTD) of XML Document.

    fDoNotUseLanguage : Boolean;

    function ParseXMLHeader (var iStr : String) : TXMLErrorCode;
    function ParseStyleSheet (var iStr : String) : TXMLErrorCode;

  public
    constructor Create;
    destructor Destroy; override;

    procedure SaveToFile (var where : TextFile);

    function ParseString (var iStr : String) : TXMLErrorCode;

  published
    property XMLVersion    : String read fXMLVersion;
    property XMLEncoding   : String read fXMLEncoding write fXMLEncoding;
    property XMLLanguage   : String read fXMLLanguage write fXMLLanguage;
    property XMLStandalone : String read fXMLStandalone write fXMLStandalone;
    property XMLOtherAttrs : TList read fXMLOtherAttrs;
    property XMLStyleSheet : String read fXMLStyleSheet write fXMLStyleSheet;
    property XMLDocType    : TXMLDocType read fXMLDocType write fXMLDocType;

end;

//-[ IMPLEMENTATION ]----------------------------------------------------------
implementation

constructor TXMLParameters.Create;
begin
  inherited Create;
  fXMLVersion := '1.0';
  fXMLEncoding := 'UTF-8';
  fXMLLanguage := 'EN';
  fXMLStandalone := 'yes';
  fXMLStyleSheet := '';
  fXMLOtherAttrs := TList.Create;
  fXMLDocType := TXMLDocType.Create;
  fDoNotUseLanguage := True;
end;

destructor TXMLParameters.Destroy;
var
  i         : Integer;
  attribute : TXMLAttribute;
begin
  for i := 0 to fXMLOtherAttrs.Count - 1 do
  begin
    attribute := fXMLOtherAttrs.Items[i];
    attribute.Free;
  end;
  fXMLOtherAttrs.Clear;
  fXMLOtherAttrs.Free;
  fXMLDocType.Free;
  inherited Destroy;
end;

//-----------------------------------------------------------------------------
// [ ParseString ]
//-----------------------------------------------------------------------------
// Parses all information, that lies outside XML Tree. This information inclu-
// des XML Header ('<?xml ... ?> ' string), stylesheet calling, if any, and
// DTD secion, if any.
//-----------------------------------------------------------------------------
function TXMLParameters.ParseString (var iStr : String) : TXMLErrorCode;
begin

  // First line of any XML document must be '<?xml ... ?>' string.
  Result := ParseXMLHeader (iStr);

  // After parsing, if everything is Ok, try to get Stylesheet and DTD info.
  if (Result = ecOk) then
  begin
    if ((iStr[1] = '<') and (iStr[2] = '?')) then
    begin
      Result := ParseStyleSheet (iStr);
    end;
    if ((iStr[1] = '<') and (iStr[2] = '!')) then
    begin
      Result := fXMLDocType.ParseString (iStr);
    end;
  end;
end;

//-----------------------------------------------------------------------------
// [ ParseXMLHeader ]
//-----------------------------------------------------------------------------
// Parses first line of XML File (stored in iStr variable) for various XML At-
// tributes, i.e. Version, Encoding, Language. If everything is Ok, this func-
// tion return ecOk and the rest of parseable string.
//-----------------------------------------------------------------------------
function TXMLParameters.ParseXMLHeader (var iStr : String) : TXMLErrorCode;
var
  tempStr  : String;        // Temporary string for various purposes.
  nextWord : String;        // Used for getting the next word from any string.
  i        : Integer;       // Cycle variable.
  flagVer  : Boolean;       // Flag of knowing the value of VERSION attribute.
  flagEnc  : Boolean;       // Flag of knowing the value of ENCODING attribute.
  flagLan  : Boolean;       // Flag of knowing the value of LANGUAGE attribute.
  flagStn  : Boolean;       // Flag of knowing the value of STANDALONE attribute.
  tempAttr : TXMLAttribute; // Temporary object for creating other parameters.
begin

  // Clearing other parameters list.
  fXMLOtherAttrs.Clear;

  // Local variables initializing.
  Result := ecOk;
  flagVer := False;
  flagEnc := False;
  flagLan := False;
  flagStn := False;
  tempStr := '';
  i := 1;

  // This cycle tries to find first <? ... ?> declaration inside XML document.
  while (not ((iStr[i] = '?') and (iStr[i + 1] = '>'))) do
  begin
    tempStr := tempStr + iStr[i];
    i := i + 1;
  end;
  tempStr := tempStr + iStr[i] + iStr[i + 1];
  Delete (iStr, 1, Length (tempStr));

  // Testing of XML header. If the string doesn't start with '<?xml ' or
  // '<? xml ' substring then the XML document is invalid.
  if ((Copy (tempStr, 1, 6) <> '<?xml ') and
      (Copy (tempStr, 1, 7) <> '<? xml ')) then
  begin
    Result := ecInvalidFileFormat;
  end
  else
  begin
    if (Copy (tempStr, 1, 6) = '<?xml ') then
    begin
      Delete (tempStr, 1, 6);
    end
    else
    begin
      Delete (tempStr, 1, 7);
    end;
  end;

  // The attempts to get values of various XML parameters (version, encoding,
  // language and so on...
  tempStr := tempStr + ' ';
  while (Pos (' ', tempStr) > 0) do
  begin
    nextWord := Copy (tempStr, 1, Pos (' ', tempStr));
    Delete (tempStr, 1, Length (nextWord));

    // Trying to set VERSION attribute.
    if (Copy (nextWord, 1, 8) = 'version=') then
    begin
      if (not flagVer) then
      begin
        flagVer := True;
        Delete (nextWord, 1, 8);
        fXMLVersion := ParseAttributeValue (nextWord);
      end
      else
      begin
        Result := ecNotValid;
      end;
      Continue;
    end;

    // Trying to get ENCODING attribute.
    if (Copy (nextWord, 1, 9) = 'encoding=') then
    begin
      if (not flagEnc) then
      begin
        flagEnc := True;
        Delete (nextWord, 1, 9);
        fXMLEncoding := ParseAttributeValue (nextWord);
      end
      else
      begin
        Result := ecNotValid;
      end;
      Continue;
    end;

    // Trying to get LANGUAGE attribute.
    if (Copy (nextWord, 1, 9) = 'language=') then
    begin
      if (not flagLan) then
      begin
        flagLan := True;
        Delete (nextWord, 1, 9);
        fXMLLanguage := ParseAttributeValue (nextWord);
      end
      else
      begin
        Result := ecNotValid;
      end;
      Continue;
    end;

    // Trying to get STANDALONE attribute.
    if (Copy (nextWord, 1, 11) = 'standalone=') then
    begin
      if (not flagStn) then
      begin
        flagStn := True;
        Delete (nextWord, 1, 11);
        fXMLStandalone := ParseAttributeValue (nextWord);
      end
      else
      begin
        Result := ecNotValid;
      end;
      Continue;
    end;

    { DONE 1 -oDushkin R.V. -cFunction : Try to create parsing cycle for yet unknown XML parameters. }
    if ((nextWord <> '?> ') and (nextWord <> '> ')) then
    begin
      tempAttr := TXMLAttribute.Create;
      Result := tempAttr.ParseString (nextWord);
      fXMLOtherAttrs.Add (tempAttr);
    end;

  end;
end;

{ DONE 5 -oDushkin R.V. -cFunction : Implement ParseStyleSheet function, which must delete <?stylesheet ... ?> section from the XML-document. }
function TXMLParameters.ParseStyleSheet (var iStr : String) : TXMLErrorCode;
begin
  Result := ecOk;
  fXMLStyleSheet := Copy (iStr, 1, Pos ('?>', iStr) + 1);
  Delete (iStr, 1, Pos ('?>', iStr) + 1);
end;

//-----------------------------------------------------------------------------
// [ SaveToFile ]
//-----------------------------------------------------------------------------
// Saves XML Header to the given file. The first line is '<?xml ... ?> defini-
// tion. Then goes StyleSheet and DOCTYPE definition (if any), and the empty
// line for aesthetic purposes.
//-----------------------------------------------------------------------------
procedure TXMLParameters.SaveToFile (var where : TextFile);
var
  i        : Integer;       // Cycling variable.
  tempStr  : String;        // Temporary string for various purposes.
  tempAttr : TXMLAttribute; // Object for pointing to other attributes.
begin

  // Form first line of XML Document. Include there four known parameters.
  if (fDoNotUseLanguage) then
  begin
    tempStr := '<?xml version="' + fXMLVersion +
                   '" encoding="' + fXMLEncoding +
                   '" standalone="' + fXMLStandalone + '"';
  end
  else
  begin
    tempStr := '<?xml version="' + fXMLVersion +
                   '" encoding="' + fXMLEncoding +
                   '" language="' + fXMLLanguage +
                   '" standalone="' + fXMLStandalone + '"';
  end;

  // Add to parameters unknowm XML parameters (stored in fXMLOtherAttrs list).
  fXMLOtherAttrs.Pack;
  for i := 0 to fXMLOtherAttrs.Count - 1 do
  begin
    tempAttr := fXMLOtherAttrs.Items[i];
    tempStr := tempStr + ' ' + tempAttr.Name + '="' + tempAttr.Value + '"';
  end;
  tempStr := tempStr + '?>';

  // Write first line to the given file.
  WriteLn (where, tempStr);

  // Write to the given file StyleSheet definition and DOCTYPE section (if any).
  if (fXMLStyleSheet <> '') then
  begin
    WriteLn (where, fXMLStyleSheet);
  end;
  fXMLDocType.SaveToFile (where);

  // Write to the given file one empty line (only for aesthetic purposes).
  WriteLn (where);
end;

//-[ END ]---------------------------------------------------------------------
end.
